import math

import pyglet
from pyglet.gl import *

from level import *

class Player:
    """Represents the player's avatar"""

    def __init__(self, level, game):
        self.level = level
        self.game = game
        self.keys = 0
        self.stars = 0
        
        self.respawn()

    def clean(self):
        """Clean data to prevent adding rounding errors"""
        self.x = math.modf(self.x)[1] + 0.5
        self.y = math.modf(self.y)[1] + 0.5
        if math.fabs(self.heading) < math.pi / 4:
            self.heading = 0
        elif math.fabs(self.heading - (math.pi / 2)) < math.pi / 4:
            self.heading = math.pi / 2
        elif math.fabs(self.heading - math.pi) < math.pi / 4:
            self.heading = math.pi
        elif math.fabs(self.heading - (3 * math.pi / 2)) < math.pi / 4:
            self.heading = 3 * math.pi / 2
            
    def interact(self):
        x = int(math.floor(self.x))
        y = int(math.floor(self.y))
        
        # One Way mechanic
        old_x = int(math.floor(self.x - math.cos(self.heading)))
        old_y = int(math.floor(self.y - math.sin(self.heading)))
        if self.level.data[old_x][old_y] == Level.FLOOR:
	        self.level.data[old_x][old_y] = 0
	        self.level.cache()
    
    def respawn(self):
        self.x = self.level.spawnX
        self.y = self.level.spawnY
        self.heading = self.level.spawnHeading
        self.cooldown = 0
        self.isMoving = False
        self.isTurningLeft = False
        self.isTurningRight = False
        self.keys = 0
        self.stars = 0

    def moveForward(self):
        if self.cooldown == 0:
            new_x = int(math.floor(self.x + math.cos(self.heading)))
            new_y = int(math.floor(self.y + math.sin(self.heading)))
            if self.level.data[new_x][new_y] == Level.WALL:
                self.game.restart()
            elif self.level.data[new_x][new_y] == Level.DOOR:
                if self.keys > 0:
                    self.level.data[new_x][new_y] = Level.FLOOR
                    self.level.cache()
                    self.keys -= 1
                    self.game.openDoor.play()
                else:
                    return
    
            self.isMoving = True
            self.cooldown = 1
            self.clean()
            self.game.footsteps.play()
    
    def turnLeft(self):
        if self.cooldown == 0:
            self.isTurningLeft = True
            self.cooldown = 1
            self.clean()
    
    def turnRight(self):
        if self.cooldown == 0:
            self.isTurningRight = True
            self.cooldown = 1
            self.clean()

    def update(self, dt):
        if self.isMoving:
            self.x += math.cos(self.heading) * 2 * dt
            self.y += math.sin(self.heading) * 2 * dt
            self.cooldown -= 2 * dt
            if self.cooldown <= 0:
                self.isMoving = False
                self.cooldown = 0
                self.clean()
                self.interact()
        elif self.isTurningLeft:
            self.heading = (self.heading + math.pi * dt) % (2 * math.pi)
            self.cooldown -= 2 * dt
            if self.cooldown <= 0:
                self.isTurningLeft = False
                self.cooldown = 0
                self.clean()
        elif self.isTurningRight:
            self.heading = (self.heading - math.pi * dt) % (2 * math.pi)
            self.cooldown -= 2 * dt
            if self.cooldown <= 0:
                self.isTurningRight = False
                self.cooldown = 0
                self.clean()
